@@ -1,11 +1,14 @@
 import os
 import re
+from collections import defaultdict
 from enum import Enum, unique
 from itertools import tee
 from pathlib import Path
 from typing import (
     Callable,
+    DefaultDict,
     Dict,
+    FrozenSet,
     Iterable,
     Iterator,
     List,
@@ -37,6 +40,12 @@ from ldb.index import index
 from ldb.index.utils import FileSystemPath, expand_indexing_paths
 from ldb.path import InstanceDir, WorkspacePath
 from ldb.storage import StorageLocation, get_storage_locations
+from ldb.transform import (
+    TransformInfo,
+    dataset_identifier_to_transform_ids,
+    get_transform_infos_from_items,
+    get_transform_mapping_dir_items,
+)
 from ldb.utils import (
     DATA_OBJ_ID_PATTERN,
     DATA_OBJ_ID_PREFIX,
@@ -45,11 +54,14 @@ from ldb.utils import (
     WORKSPACE_DATASET_PREFIX,
     format_dataset_identifier,
     get_hash_path,
+    json_dumps,
     parse_data_object_hash_identifier,
     parse_dataset_identifier,
 )
 from ldb.workspace import collection_dir_to_object, load_workspace_dataset
 
+TransformInfoMapping = Dict[str, FrozenSet[TransformInfo]]
+
 
 @unique
 class ArgType(Enum):
@@ -88,6 +100,101 @@ def expands_to_workspace(urlpath: str) -> bool:
     return False
 
 
+ArgProcessingFunc = Callable[[Path, Sequence[str]], AddInput]
+
+
+def paths_to_dataset(
+    ldb_dir: Path,
+    paths: Sequence[str],
+    collection_ops: Iterable[OpDef],
+    warn: bool = True,
+    include_transforms: bool = True,
+    arg_processing_func: Optional[ArgProcessingFunc] = None,
+) -> Tuple[Iterator[Tuple[str, str]], Optional[TransformInfoMapping]]:
+    if arg_processing_func is None:
+        arg_processing_func = process_args_for_ls
+    if not paths:
+        paths = ["ws:."]
+    if include_transforms:
+        transform_infos = paths_to_transforms(ldb_dir, paths)
+    else:
+        transform_infos = None
+    data_object_hashes, annotation_hashes, message = arg_processing_func(
+        ldb_dir,
+        paths,
+    )
+    if message:
+        print(message)
+        print()
+    collection = apply_queries(
+        ldb_dir,
+        data_object_hashes,
+        annotation_hashes,
+        collection_ops,
+        warn=warn,
+    )
+    return collection, transform_infos
+
+
+def paths_to_transforms(
+    ldb_dir: Path,
+    paths: Sequence[str],
+) -> Dict[str, FrozenSet[TransformInfo]]:
+    transform_infos = _paths_to_transform_id_sets(ldb_dir, paths)
+    return get_transform_infos_from_items(ldb_dir, transform_infos.items())
+
+
+def paths_to_transform_ids(
+    ldb_dir: Path,
+    paths: Sequence[str],
+    workspace_path: Optional[Path] = None,
+) -> Dict[str, Tuple[str, ...]]:
+    transforms_id_sets = _paths_to_transform_id_sets(
+        ldb_dir,
+        paths,
+    )
+    if workspace_path is not None:
+        for data_obj_id, transforms_id_set in _paths_to_transform_id_sets(
+            ldb_dir,
+            paths,
+        ).items():
+            try:
+                transforms_id_sets[data_obj_id] |= transforms_id_set
+            except KeyError:
+                transforms_id_sets[data_obj_id] = transforms_id_set
+    return {
+        data_obj_id: tuple(transform_id_set)
+        for data_obj_id, transform_id_set in transforms_id_sets.items()
+    }
+
+
+def _paths_to_transform_id_sets(
+    ldb_dir: Path,
+    paths: Sequence[str],
+) -> DefaultDict[str, Set[str]]:
+    arg_type = get_arg_type(paths)
+    transform_infos: DefaultDict[str, Set[str]] = defaultdict(set)
+    separate_infos: List[Iterable[Tuple[str, List[str]]]] = []
+    if arg_type == ArgType.DATASET:
+        for dataset_identifier in paths:
+            transform_obj = dataset_identifier_to_transform_ids(
+                ldb_dir,
+                dataset_identifier,
+            )
+            separate_infos.append(transform_obj.items())
+    elif arg_type == ArgType.WORKSPACE_DATASET:
+        paths = [re.sub(r"^ws:", "", p) for p in paths]
+        for ws_path in paths:
+            info_items = get_transform_mapping_dir_items(
+                Path(ws_path) / WorkspacePath.TRANSFORM_MAPPING,
+            )
+            separate_infos.append(list(info_items))
+    for infos in separate_infos:
+        for data_object_id, transform_ids in infos:
+            transform_infos[data_object_id].update(transform_ids)
+    return transform_infos
+
+
 def process_args_for_add(
     ldb_dir: Path,
     paths: Sequence[str],
@@ -228,6 +335,7 @@ def add(
     paths: Sequence[str],
     query_args: Iterable[OpDef],
     ldb_dir: Optional[Path] = None,
+    warn: bool = False,
 ) -> List[Tuple[str, str]]:
     if not paths:
         if not query_args:
@@ -241,7 +349,9 @@ def add(
     workspace_path = Path(os.path.normpath(workspace_path))
     ds_name = load_workspace_dataset(workspace_path).dataset_name
     collection_dir_path = workspace_path / WorkspacePath.COLLECTION
+    transform_dir_path = workspace_path / WorkspacePath.TRANSFORM_MAPPING
     collection_dir_path.mkdir(exist_ok=True)
+    transform_dir_path.mkdir(exist_ok=True)
 
     data_object_hashes, annotation_hashes, message = process_args_for_add(
         ldb_dir,
@@ -256,8 +366,9 @@ def add(
         data_object_hashes,
         annotation_hashes,
         query_args,
-        warn=False,
+        warn=warn,
     )
+
     print("Adding to working dataset...")
 
     collection_list = list(collection)
@@ -265,9 +376,22 @@ def add(
         collection_dir_path,
         collection_list,
     )
-
+    transform_obj = paths_to_transform_ids(
+        ldb_dir,
+        paths,
+        workspace_path=workspace_path,
+    )
+    transform_data = [
+        (data_obj_id, json_dumps(transforms))
+        for data_obj_id, transforms in transform_obj.items()
+    ]
+    num_transforms = add_to_collection_dir(
+        transform_dir_path,
+        transform_data,
+    )
     ds_ident = format_dataset_identifier(ds_name)
     print(f"Added {num_data_objects} data objects to {ds_ident}")
+    print(f"Updated {num_transforms} transforms")
     return collection_list
 
 
