@@ -2,6 +2,7 @@ from argparse import ArgumentParser, Namespace
 from pathlib import Path
 from typing import TYPE_CHECKING, Iterable
 
+from ldb.cli_utils import json_bool
 from ldb.commit import commit
 from ldb.core import get_ldb_instance
 
@@ -10,7 +11,13 @@ if TYPE_CHECKING:
 
 
 def commit_command(options: Namespace) -> None:
-    commit(get_ldb_instance(), Path("."), options.dataset, options.message)
+    commit(
+        get_ldb_instance(),
+        Path("."),
+        dataset_identifier=options.dataset,
+        message=options.message,
+        auto_pull=options.auto_pull,
+    )
 
 
 def add_parser(
@@ -36,4 +43,13 @@ def add_parser(
         default="",
         help="A message about this commit",
     )
+    parser.add_argument(
+        "--auto-pull",
+        default=None,
+        choices=(True, False),
+        type=json_bool,
+        const=True,
+        nargs="?",
+        help="Set the auto-pull field on this dataset",
+    )
     parser.set_defaults(func=commit_command)
