@@ -5,8 +5,6 @@ from typing import TYPE_CHECKING, Iterable
 from ldb.cli_utils import add_instantiate_arguments
 from ldb.core import get_ldb_instance
 from ldb.instantiate import instantiate
-from ldb.utils import format_dataset_identifier
-from ldb.workspace import load_workspace_dataset
 
 if TYPE_CHECKING:
     from argparse import _SubParsersAction
@@ -14,13 +12,8 @@ if TYPE_CHECKING:
 
 def instantiate_command(options: Namespace) -> None:
     ldb_dir = get_ldb_instance()
-    workspace_path = Path(".")
-    workspace_ds = load_workspace_dataset(workspace_path)
-    ds_ident = format_dataset_identifier(workspace_ds.dataset_name)
-    print(f"Instantiating working dataset {ds_ident}...\n")
     result = instantiate(
         ldb_dir,
-        workspace_path,
         Path(options.target_dir),
         options.paths,
         options.query_args,
@@ -28,6 +21,7 @@ def instantiate_command(options: Namespace) -> None:
         force=options.force,
         apply=options.apply,
         warn=False,
+        params=dict(options.params),
     )
     print(
         "Copied dataset to workspace.\n"
