@@ -1,9 +1,11 @@
 import getpass
 import os
 from pathlib import Path
+from typing import Optional
 
 from ldb.dataset import CommitInfo, Dataset, DatasetVersion
 from ldb.path import InstanceDir, WorkspacePath
+from ldb.transform import save_transform_object
 from ldb.utils import (
     DATASET_PREFIX,
     current_time,
@@ -27,6 +29,7 @@ def commit(
     workspace_path: Path,
     dataset_identifier: str = "",
     message: str = "",
+    auto_pull: Optional[bool] = None,
 ) -> None:
     workspace_path = Path(os.path.normpath(workspace_path))
     workspace_ds = load_workspace_dataset(workspace_path)
@@ -66,6 +69,7 @@ def commit(
         collection_obj_bytes,
         overwrite_existing=False,
     )
+    transform_hash = save_transform_object(ldb_dir, workspace_path)
 
     curr_time = current_time()
     username = getpass.getuser()
@@ -80,16 +84,19 @@ def commit(
             created=curr_time,
             versions=[],
         )
+    auto_pull = workspace_ds.auto_pull if auto_pull is None else auto_pull
     dataset_version = DatasetVersion(
         version=len(dataset.versions) + 1,
         parent=workspace_ds.parent,
         collection=collection_hash,
+        transform_mapping_id=transform_hash,
         tags=workspace_ds.tags.copy(),
         commit_info=CommitInfo(
             created_by=username,
             commit_time=curr_time,
             commit_message=message,
         ),
+        auto_pull=auto_pull,
     )
     dataset_version_bytes = json_dumps(dataset_version.format()).encode()
     dataset_version_hash = hash_data(dataset_version_bytes)
@@ -107,6 +114,7 @@ def commit(
     workspace_ds.staged_time = curr_time
     workspace_ds.parent = dataset_version_hash
     workspace_ds.dataset_name = dataset_name
+    workspace_ds.auto_pull = auto_pull
     write_data_file(
         workspace_path / WorkspacePath.DATASET,
         json_dumps(workspace_ds.format()).encode(),
