@@ -1,3 +1,4 @@
+from functools import partial
 from itertools import tee
 from pathlib import Path
 from typing import Iterable, Iterator, List, Optional, Sequence, Tuple, Union
@@ -6,7 +7,7 @@ from ldb.add import process_args_for_ls
 from ldb.dataset import apply_queries, get_annotations, get_data_object_metas
 from ldb.func_utils import apply_optional
 from ldb.op_type import OpType
-from ldb.query.search import get_search_func
+from ldb.query.search import SearchFunc, get_search_func
 from ldb.typing import JSONDecoded
 
 EvaluateResult = Union[
@@ -20,13 +21,16 @@ def evaluate(
     ldb_dir: Path,
     paths: Sequence[str],
     collection_ops: Iterable[Tuple[str, str]],
+    warn: bool = True,
 ) -> Iterator[EvaluateResult]:
-    annotation_query, file_query, collection_ops = process_query_args(
+    search, file_search, collection_ops = process_query_args(
         collection_ops,
     )
-    search = apply_optional(get_search_func, annotation_query)
-    file_search = apply_optional(get_search_func, file_query)
-    data_object_hashes, annotation_hashes, _ = process_args_for_ls(
+    (
+        data_object_hashes,
+        annotation_hashes,
+        _,
+    ) = process_args_for_ls(  # pylint: disable=duplicate-code
         ldb_dir,
         paths,
     )
@@ -35,6 +39,7 @@ def evaluate(
         data_object_hashes,
         annotation_hashes,
         collection_ops,
+        warn=warn,
     )
     collection1, collection2 = tee(collection)
     data_object_hashes1, data_object_hashes2 = tee(d for d, _ in collection1)
@@ -45,13 +50,13 @@ def evaluate(
         search_results = []
         if file_search is not None:
             search_results.append(
-                file_search(
+                file_search(  # type: ignore[arg-type]
                     get_data_object_metas(ldb_dir, data_object_hashes1),
                 ),
             )
         if search is not None:
             search_results.append(
-                search(get_annotations(ldb_dir, annotation_hashes)),
+                search(get_annotations(ldb_dir, annotation_hashes)),  # type: ignore[arg-type] # noqa: E501
             )
 
     result: Iterator[EvaluateResult] = zip(  # type: ignore[assignment]
@@ -63,20 +68,29 @@ def evaluate(
 
 def process_query_args(
     query_args: Iterable[Tuple[str, str]],
-) -> Tuple[Optional[str], Optional[str], List[Tuple[str, str]]]:
+    warn: bool = True,
+) -> Tuple[Optional[SearchFunc], Optional[SearchFunc], List[Tuple[str, str]]]:
     query_args = list(query_args)
-    search_args = {}
-    possible_search_op_types = {OpType.ANNOTATION_QUERY, OpType.FILE_QUERY}
-    while query_args and possible_search_op_types:
-        op_type, arg = query_args[-1]
-        if op_type in possible_search_op_types:
-            query_args.pop()
-            search_args[op_type] = arg
-            possible_search_op_types.remove(op_type)
-        else:
-            break
-    return (
-        search_args.get(OpType.ANNOTATION_QUERY),
-        search_args.get(OpType.FILE_QUERY),
-        query_args,
-    )
+    search = None
+    file_search = None
+    if not query_args:
+        return search, file_search, query_args
+    op_type, arg = query_args.pop()
+    if op_type == OpType.ANNOTATION_QUERY:
+        search = apply_optional(
+            partial(get_search_func, use_custom=True, warn=warn),
+            arg,
+        )
+    elif op_type == OpType.JP_ANNOTATION_QUERY:
+        search = apply_optional(
+            partial(get_search_func, use_custom=False, warn=warn),
+            arg,
+        )
+    elif op_type == OpType.FILE_QUERY:
+        file_search = apply_optional(
+            partial(get_search_func, warn=warn),
+            arg,
+        )
+    else:
+        query_args.append((op_type, arg))
+    return search, file_search, query_args
