@@ -27,13 +27,18 @@ from fsspec.utils import get_protocol
 from ldb.data_formats import Format
 from ldb.exceptions import IndexingException, NotAStorageLocationError
 from ldb.fs import posix_path as fsp
-from ldb.fs.utils import cp_file_any_fs, has_protocol
+from ldb.fs.utils import (
+    cp_file_any_fs,
+    get_file_hash,
+    get_modified_time,
+    has_protocol,
+)
 from ldb.func_utils import apply_optional
 from ldb.storage import StorageLocation, get_filesystem
 from ldb.typing import JSONDecoded
 from ldb.utils import (
+    StrEnum,
     format_datetime,
-    get_file_hash,
     get_filetype,
     get_first,
     get_fsspec_path_suffix,
@@ -60,6 +65,11 @@ class FileSystemPath(NamedTuple):
     path: str
 
 
+class AnnotMergeStrategy(StrEnum):
+    MERGE = "merge"
+    REPLACE = "replace"
+
+
 @dataclass
 class IndexingConfig:
     save_data_object_path_info: bool
@@ -176,6 +186,7 @@ def group_indexing_paths_by_type(
 
 def expand_dir_paths(
     paths: Iterable[str],
+    storage_locations: Sequence[StorageLocation],
 ) -> Dict[AbstractFileSystem, List[str]]:
     for path in paths:
         path, num = re.subn(ENDING_DOUBLE_STAR_RE, "", path)
@@ -189,7 +200,12 @@ def expand_dir_paths(
     path_sets: Dict[AbstractFileSystem, Set[str]] = {}
     for path in paths:
         if not is_hidden_fsspec_path(path):
-            fs = fsspec.filesystem(get_protocol(path))
+            protocol = get_protocol(path)
+            fs_cls = fsspec.get_filesystem_class(protocol)
+            path = fs_cls._strip_protocol(  # pylint: disable=protected-access
+                path,
+            )
+            fs = get_filesystem(path, protocol, storage_locations)
             fs_paths = path_sets.setdefault(fs, set())
             for p in fs.expand_path(paths):
                 if not is_hidden_fsspec_path(p) and fs.isdir(p):
@@ -198,7 +214,9 @@ def expand_dir_paths(
     for fs, fs_path_set in path_sets.items():
         paths = sorted(fs_path_set)
         for i in range(len(paths) - 1):
-            if paths[i + 1].startswith(paths[i]):
+            if (paths[i + 1].rstrip("/") + "/").startswith(
+                paths[i].rstrip("/") + "/",
+            ):
                 raise IndexingException(
                     f"Paths passed with the {Format.INFER} format should "
                     "match non-overlapping directories. Found overlapping "
@@ -454,13 +472,21 @@ def construct_data_object_meta(
     prev_meta: Dict[str, Any],
     current_timestamp: str,
     tags: Collection[str] = (),
-) -> DataObjectMeta:
+) -> Tuple[bool, DataObjectMeta]:
     # TODO: create dataclass for data object meta
     fs_info = fs.info(path)
 
     last_indexed_by: str = getpass.getuser()
+
     atime = datetime_fs_info(fs_info, "atime", "accessed", "time")
-    mtime = datetime_fs_info(fs_info, "mtime", "modified")
+    if atime is None and fs.protocol == "file":
+        stat_result = os.stat(
+            fs._strip_protocol(path),  # pylint: disable=protected-access
+            follow_symlinks=False,
+        )
+        atime = timestamp_to_datetime(stat_result.st_atime)
+
+    mtime = get_modified_time(fs, path)
     ctime = datetime_fs_info(fs_info, "ctime", "created")
     size: int = fs_info.get("size", 0)
     mode: int = fs_info.get("mode", 0)
@@ -489,7 +515,10 @@ def construct_data_object_meta(
     }
     if path_info not in alternate_paths:
         alternate_paths.append(path_info)
-    return {
+        found_new_path = True
+    else:
+        found_new_path = False
+    return found_new_path, {
         "type": filetype,
         "first_indexed": first_indexed,
         "last_indexed": current_timestamp,
