@@ -4,9 +4,11 @@ import shutil
 import subprocess
 import tempfile
 from concurrent.futures import ThreadPoolExecutor
+from contextlib import suppress
 from dataclasses import dataclass, field
 from pathlib import Path
 from typing import (
+    Any,
     Collection,
     Iterable,
     List,
@@ -14,7 +16,6 @@ from typing import (
     NamedTuple,
     Optional,
     Sequence,
-    Set,
     Tuple,
     Union,
     cast,
@@ -23,20 +24,17 @@ from typing import (
 import jmespath
 from funcy.objects import cached_property
 
-from ldb.add import process_args_for_delete
+from ldb.add import TransformInfoMapping, paths_to_dataset
 from ldb.data_formats import INSTANTIATE_FORMATS, Format
-from ldb.dataset import OpDef, apply_queries_to_collection, get_annotation
+from ldb.dataset import OpDef, get_annotation
 from ldb.exceptions import LDBException
-from ldb.fs.utils import FSProtocol, first_protocol
+from ldb.fs.utils import FSProtocol, first_protocol, unstrip_protocol
+from ldb.index.inferred import InferredParamConfig
+from ldb.params import ParamConfig
 from ldb.path import InstanceDir, WorkspacePath
 from ldb.progress import get_progressbar
 from ldb.storage import StorageLocation, get_filesystem, get_storage_locations
-from ldb.transform import (
-    DEFAULT,
-    TransformInfo,
-    TransformType,
-    get_transform_infos_from_dir,
-)
+from ldb.transform import DEFAULT, TransformInfo, TransformType
 from ldb.typing import JSONDecoded, JSONObject
 from ldb.utils import (
     DATA_OBJ_ID_PREFIX,
@@ -46,10 +44,7 @@ from ldb.utils import (
     make_target_dir,
     write_data_file,
 )
-from ldb.workspace import (
-    collection_dir_to_object,
-    ensure_path_is_empty_workspace,
-)
+from ldb.workspace import ensure_path_is_empty_workspace
 
 
 @dataclass
@@ -67,6 +62,7 @@ class InstConfig:
     transform_infos: Mapping[str, Collection[TransformInfo]] = field(
         default_factory=dict,
     )
+    params: Mapping[str, Any] = field(default_factory=dict)
 
 
 class InstantiateResult(NamedTuple):
@@ -78,7 +74,6 @@ class InstantiateResult(NamedTuple):
 
 def instantiate(
     ldb_dir: Path,
-    workspace_path: Path,
     dest: Path,
     paths: Sequence[str] = (),
     query_args: Iterable[OpDef] = (),
@@ -87,99 +82,108 @@ def instantiate(
     apply: Sequence[str] = (),
     make_parent_dirs: bool = False,
     warn: bool = True,
-) -> InstantiateResult:
-    if fmt not in INSTANTIATE_FORMATS:
-        raise ValueError(f"Not a valid instantiation format: {fmt}")
-
-    make_target_dir(dest, parents=make_parent_dirs)
-    data_object_hashes: Set[str] = set(
-        process_args_for_delete(
-            ldb_dir,
-            paths,
-        ),
-    )
-    orig_collection = collection_dir_to_object(
-        workspace_path / WorkspacePath.COLLECTION,
-    )
-    collection = {
-        d: a if a is not None else ""
-        for d, a in orig_collection.items()
-        if d in data_object_hashes
-    }
-    collection = dict(
-        apply_queries_to_collection(
-            ldb_dir,
-            collection.items(),
-            query_args,
-            warn=warn,
-        ),
-    )
-    return instantiate_collection(
-        ldb_dir,
-        workspace_path,
-        collection,
-        dest,
-        fmt,
-        force,
-        apply,
-    )
-
-
-def instantiate_collection(
-    ldb_dir: Path,
-    workspace_path: Path,
-    collection: Mapping[str, Optional[str]],
-    dest: Path,
-    fmt: str = Format.BARE,
-    force: bool = False,
-    apply: Sequence[str] = (),
-    clean: bool = True,
+    params: Optional[Mapping[str, str]] = None,
 ) -> InstantiateResult:
     try:
         fmt = INSTANTIATE_FORMATS[fmt]
     except KeyError as exc:
         raise ValueError(f"Not a valid instantiation format: {fmt}") from exc
+    if params is None:
+        params = {}
+
+    if fmt == Format.INFER:
+        processed_params = InferredParamConfig().process_params(params, fmt)
+    else:
+        processed_params = ParamConfig().process_params(params, fmt)
+
+    make_target_dir(dest, parents=make_parent_dirs)
+    collection, transform_infos = paths_to_dataset(
+        ldb_dir,
+        paths,
+        query_args,
+        warn=warn,
+        include_transforms=fmt in (Format.STRICT, Format.BARE),
+    )
+    return instantiate_collection(
+        ldb_dir,
+        dict(collection),
+        dest,
+        transform_infos=transform_infos,
+        fmt=fmt,
+        force=force,
+        apply=apply,
+        params=processed_params,
+    )
+
+
+def instantiate_collection(
+    ldb_dir: Path,
+    collection: Mapping[str, Optional[str]],
+    dest: Path,
+    transform_infos: Optional[TransformInfoMapping] = None,
+    fmt: str = Format.BARE,
+    force: bool = False,
+    apply: Sequence[str] = (),
+    clean: bool = True,
+    params: Optional[Mapping[str, Any]] = None,
+    tmp_dir: Optional[Union[str, Path]] = None,
+) -> InstantiateResult:
+    try:
+        fmt = INSTANTIATE_FORMATS[fmt]
+    except KeyError as exc:
+        raise ValueError(f"Not a valid instantiation format: {fmt}") from exc
+    default_tmp = dest / WorkspacePath.TMP
+    if tmp_dir is None:
+        tmp_dir = default_tmp
+    tmp_dir = os.path.abspath(tmp_dir)
+    if params is None:
+        params = {}
     # fail fast if workspace is not empty
     if clean and dest.exists():
         ensure_path_is_empty_workspace(dest, force)
     storage_locations = get_storage_locations(ldb_dir)
-    if fmt in (Format.STRICT, Format.BARE):
-        transform_infos = get_transform_infos_from_dir(
-            ldb_dir,
-            workspace_path / WorkspacePath.TRANSFORM_MAPPING,
-        )
-    else:
-        transform_infos = {}
     dest.mkdir(exist_ok=True)
 
-    tmp_dir_base = workspace_path.absolute() / WorkspacePath.TMP
-    tmp_dir_base.mkdir(exist_ok=True)
-    with tempfile.TemporaryDirectory(dir=tmp_dir_base) as final_tmp_dir:
-        with tempfile.TemporaryDirectory(dir=tmp_dir_base) as raw_tmp_dir:
-            config = InstConfig(
-                ldb_dir=ldb_dir,
-                dest_dir=final_tmp_dir,
-                intermediate_dir=raw_tmp_dir,
-                storage_locations=storage_locations,
-                transform_infos=transform_infos,
-            )
-            result = instantiate_collection_directly(
-                config,
-                collection,
-                fmt,
-            )
-        if apply:
-            paths = [str(ldb_dir / InstanceDir.USER_FILTERS)]
-            apply_transform(apply, final_tmp_dir, os.fspath(dest), paths=paths)
-        else:
-            # check again to make sure nothing was added while writing to the
-            # temporary location
-            if clean:
-                ensure_path_is_empty_workspace(dest, force)
-            dest_str = os.fspath(dest)
-            for path in Path(final_tmp_dir).iterdir():
-                os.replace(os.fspath(path), os.path.join(dest_str, path.name))
-
+    try:
+        os.makedirs(tmp_dir, exist_ok=True)
+        with tempfile.TemporaryDirectory(dir=tmp_dir) as final_tmp_dir:
+            with tempfile.TemporaryDirectory(dir=tmp_dir) as raw_tmp_dir:
+                config = InstConfig(
+                    ldb_dir=ldb_dir,
+                    dest_dir=final_tmp_dir,
+                    intermediate_dir=raw_tmp_dir,
+                    storage_locations=storage_locations,
+                    transform_infos=transform_infos or {},
+                    params=params,
+                )
+                result = instantiate_collection_directly(
+                    config,
+                    collection,
+                    fmt,
+                )
+            if apply:
+                paths = [str(ldb_dir / InstanceDir.USER_FILTERS)]
+                apply_transform(
+                    apply,
+                    final_tmp_dir,
+                    os.fspath(dest),
+                    paths=paths,
+                )
+            else:
+                # check again to make sure nothing was added while writing to
+                # the temporary location
+                if clean:
+                    ensure_path_is_empty_workspace(dest, force)
+                dest_str = os.fspath(dest)
+                for path in Path(final_tmp_dir).iterdir():
+                    os.replace(
+                        os.fspath(path),
+                        os.path.join(dest_str, path.name),
+                    )
+    finally:
+        with suppress(OSError):
+            os.rmdir(default_tmp)
+            os.rmdir(os.path.dirname(default_tmp))
     return result
 
 
@@ -290,7 +294,13 @@ class InstItem:
         os.makedirs(os.path.split(self.data_object_dest)[0], exist_ok=True)
         dest = self.data_object_dest
 
-        fs.get_file(path, dest)
+        try:
+            fs.get_file(path, dest)
+        except FileNotFoundError as exc:
+            full_path = unstrip_protocol(fs, path)
+            if exc.args and full_path in exc.args:
+                raise
+            raise FileNotFoundError(full_path) from exc
         return dest
 
     def copy_files(self) -> ItemCopyResult:
@@ -426,8 +436,16 @@ class AnnotationOnlyInstItem(RawPairInstItem):
             self.config.ldb_dir,
             self.annotation_hash,
         )
+        fs_protocol: FSProtocol = self.data_object_meta["fs"]["protocol"]
+        protocol: str = first_protocol(fs_protocol)
+        path: str = self.data_object_meta["fs"]["path"]
+        fs = get_filesystem(path, protocol, self.config.storage_locations)
+        path = unstrip_protocol(fs, path)
         annotation = {
-            "ldb_meta": {"data_object_id": self.data_object_hash},
+            "data-object-info": {
+                "md5": self.data_object_hash,
+                "path": path,
+            },
             "annotation": annotation,
         }
         return annotation
@@ -439,22 +457,30 @@ class AnnotationOnlyInstItem(RawPairInstItem):
 @dataclass
 class InferInstItem(RawPairInstItem):
     annotation_hash: str
+    label_key: Sequence[str]
+    base_label: str = ""
 
     @cached_property
     def base_dest(self) -> str:
         parts: List[str] = []
+        key: str
+        label: Union[JSONObject, str]
         try:
-            label = self.annotation_content["label"]  # type: ignore[index, call-overload] # noqa: E501
+            label = self.annotation_content  # type: ignore[assignment]
+            for key in self.label_key:
+                label = label[key]  # type: ignore[index]
         except Exception as exc:
+            # TODO: convert self.label_key to jmespath expression for message
             raise LDBException(
                 "Annotations for tensorflow-inferred format should contain "
-                '"label" key',
+                f"key: {self.label_key}",
             ) from exc
-        key: str
-        while isinstance(label, dict):
-            key, label = next(iter(label.items()))
-            parts.append(key)
-        parts.append(label)
+
+        if label != self.base_label:
+            while isinstance(label, dict):
+                key, label = next(iter(label.items()))  # type: ignore[assignment] # noqa: E501
+                parts.append(key)
+            parts.append(label)
         return os.path.join(
             self.dest_dir,
             *parts,
@@ -589,6 +615,8 @@ def copy_infer(
     config: InstConfig,
     collection: Mapping[str, Optional[str]],
 ) -> InstantiateResult:
+    label_key: Sequence[str] = config.params.get("label-key", ["label"])
+    base_label: str = config.params.get("base-label", "")
     for data_object_hash, annotation_hash in collection.items():
         if not annotation_hash:
             raise LDBException(
@@ -607,6 +635,8 @@ def copy_infer(
                 config,
                 data_object_hash,
                 annotation_hash,
+                label_key=label_key,
+                base_label=base_label,
             ),
         )
     data_obj_paths, _ = instantiate_items(items)
