@@ -2,10 +2,13 @@ __all__ = ["main"]
 
 import sys
 import traceback
+import warnings
 from typing import List, Optional
 
 from ldb.cli import get_main_parser
 from ldb.exceptions import LDBException
+from ldb.params import InvalidParamError
+from ldb.warnings import SimpleWarningHandler
 
 MIN_PYTHON_VERSION_INFO = (3, 7)
 MIN_PYTHON_VERSION = ".".join(map(str, MIN_PYTHON_VERSION_INFO))
@@ -25,14 +28,16 @@ def main(argv: Optional[List[str]] = None) -> int:
         except AttributeError:
             main_parser.print_usage()
             return 1
-        func(options)
+        with warnings.catch_warnings():
+            warnings.showwarning = SimpleWarningHandler.showwarning
+            func(options)
     except Exception as exc:  # pylint: disable=broad-except
         return handle_exception(exc, options.verbose)
     return 0
 
 
 def handle_exception(exception: BaseException, verbose: int = 0) -> int:
-    if isinstance(exception, LDBException):
+    if isinstance(exception, (LDBException, InvalidParamError)):
         print("ERROR:", exception, file=sys.stderr)
     else:
         print(
